package net.sf.basedb.clients.web.extensions.list;

import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import net.sf.basedb.clients.web.util.HTML;
import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.snapshot.AnnotationSnapshot;
import net.sf.basedb.core.snapshot.AnnotationTypeFilter;
import net.sf.basedb.util.Enumeration;
import net.sf.basedb.util.filter.Filter;

/**
	Parent item column that load annotation values from the parent items.
	@author nicklas
	@since 3.16
*/
public class ParentItemAnnotationColumn 
	extends ParentItemColumn
{

	private final AnnotationType at;
	private final Filter<AnnotationSnapshot> atFilter;
	
	ParentItemAnnotationColumn(int index, String id, Filter<Annotatable> filter, ParentItemHelper helper, AnnotationType at)
	{
		super(index, id, filter, helper);
		this.at = at;
		this.atFilter = new AnnotationTypeFilter(at);
		if (at.isEnumeration())
		{
			Enumeration<String, String> annotationEnum = new Enumeration<String, String>();
			annotationEnum.add("", "-none-");
			List<?> values = at.getValues();
			for (Object value : values)
			{
				String encoded = HTML.encodeTags(value.toString());
				annotationEnum.add(encoded, encoded);
			}
			setEnumeration(annotationEnum);
		}
	}
	
	@Override
	public Object getValue(ParentItemHelper helper, Annotatable item) 
	{
		Set<Annotatable> parents = getParentItems(item);
		if (parents.size() == 0) return null;
		
		Set<Object> values = new TreeSet<>();
		for (Annotatable p : parents)
		{
			for (AnnotationSnapshot ss : helper.manager.findAnnotations(helper.dc, p, atFilter, true, false))
			{
				values.addAll(ss.getActualValues());
			}
		}
		return values.size() == 0 ? null : values;
	}

}
