/*
	$Id: ColumnHeaders.java 6898 2015-05-12 09:27:08Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.table;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

/**
	Generate column headers for the columns that has been defined for this table.
	This tag has no attributes and should be inside a &gt;

	<p>
	<b>Syntax:</b>
	<pre>
	&lt;tbl:headerrow&gt;
      &lt;tbl:columnheaders /&gt;
   &lt;/tbl:headerrow&gt;
</pre>


	@author Nicklas
	@since 3.1
*/
public class ColumnHeaders
	extends TagSupport
{

	private static final long serialVersionUID = 8228986623221072939L;

	private Table table;
	
	/*
		Package methods
	*/
	Table getTable()
	{
		return table;
	}

	/*
		Display methods
	*/
	@Override
	public int doStartTag()
		throws JspException
	{
		table = (Table)findAncestorWithClass(this, Table.class);
		try
		{
			int numColumns = table.writeColumns();
			Tag parent = getParent();
			if (parent instanceof HeaderRow)
			{
				HeaderRow hr = (HeaderRow)parent;
				hr.addHeaderColumns(numColumns);
			}
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return SKIP_BODY;
	}

}

