/*
	$Id$

	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.util.extensions.ClientContext;
import net.sf.basedb.util.extensions.ExtensionsFilter;
import net.sf.basedb.util.extensions.ExtensionsInvoker;
import net.sf.basedb.util.extensions.Registry;

/**
	Utility class for the "services" extension point. While part of the web client code
	we need to place it in the core to get access to special {@link ServiceSessionControl}
	item that can be used by the service extensions to access BASE.

	@author nicklas
	@since 3.3
*/
public final class ServicesUtil 
{

	/**
		Create an extensions invoker for the services extensions. The client
		context is created with special SessionControl that gives the extension
		permission to impersonate another user.
	*/
	public static ExtensionsInvoker<?> useExtensions(ExtensionsFilter filter)
	{
		Registry registry = Application.getExtensionsManager().getRegistry();
		SessionControl sc = new ServiceSessionControl();
		ClientContext context = new ClientContext(sc);
		ExtensionsInvoker<?> invoker = registry.useExtensions(context, filter, "net.sf.basedb.clients.web.services");
		return invoker;
	}
	
}
