/**
	$Id: BioMaterialEventSourceData.java 6468 2014-06-10 11:27:02Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	Data class for holding information about a parent biomaterial
	that was used in an event that created a child biomaterial or
	a physical bioassay. Each event may only include the same parent
	biomaterial once.

	@author Nicklas
	@since 3.0
	@see <a href="../../../../../../html/developer/api/data_api.biomaterials.html">Developer documentation: Biomaterials LIMS</a>
	@base.modified $Date: 2014-06-10 13:27:02 +0200 (Tue, 10 Jun 2014) $
	@hibernate.class table="`BioMaterialEventSources2`" lazy="true"
*/
public class BioMaterialEventSourceData
	extends BasicData
{

	public BioMaterialEventSourceData()
	{}
	
	private BioMaterialData bioMaterial;
	/**
		Get the parent biomaterial that was used in the event.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`biomaterial_id`" not-null="true" unique-key="BioMaterialEventSource_uniqueparent"
	*/
	public BioMaterialData getBioMaterial()
	{
		return bioMaterial;
	}
	public void setBioMaterial(BioMaterialData bioMaterial)
	{
		this.bioMaterial = bioMaterial;
	}

	private BioMaterialEventData event;
	/**
		Get the event that created something from the parent biomaterial.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`event_id`" not-null="true" unique-key="BioMaterialEventSource_uniqueparent"
	*/
	public BioMaterialEventData getEvent()
	{
		return event;
	}
	public void setEvent(BioMaterialEventData event)
	{
		this.event = event;
	}

	private Float usedQuantity;
	/**
		Get the used quantity from the parent biomaterial. This is only
		meaningful if the parent is a {@link MeasuredBioMaterialData} and
		is ignored otherwise.
		@hibernate.property column="`used_quantity`" not-null="false" type="float"
	*/
	public Float getUsedQuantity()
	{
		return usedQuantity;
	}
	public void setUsedQuantity(Float usedQuantity)
	{
		this.usedQuantity = usedQuantity;
	}
	
	private int position = 1;
	/**
		Get the position of the parent biomaterial as it was used in the event.
		The meaning of this depends on the type of event. Some events may not
		use this at all. Multiple parent items may use the same position. 
		@hibernate.property column="`position`" type="int" not-null="true"
	*/
	public int getPosition()
	{
		return position;
	}
	public void setPosition(int position)
	{
		this.position = position;
	}

	
}
