/*
	$Id: DataCubeFilterData.java 6468 2014-06-10 11:27:02Z nicklas $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	This class holds information about a single filter in a data cube.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.DataCubeFilter
	@see <a href="../../../../../../html/developer/api/data_api.experiments.html">Developer documentation: Experiments and analysis</a>
	@base.modified $Date: 2014-06-10 13:27:02 +0200 (Tue, 10 Jun 2014) $
	@hibernate.class table="`DataCubeFilters`" lazy="true"
*/
public class DataCubeFilterData
	extends BasicData
{
	public DataCubeFilterData()
	{}

	private DataCubeData dataCube;
	/**
		Get the data cube this filter belongs to.
		@hibernate.many-to-one column="`datacube_id`" not-null="true" outer-join="false" 
			update="false" unique-key="DataCubeFilter_uniquefilter"
	*/
	public DataCubeData getDataCube()
	{
		return dataCube;
	}
	public void setDataCube(DataCubeData dataCube)
	{
		this.dataCube = dataCube;
	}

	private short filterNo;
	/**
		The number of this filter in the data cube. Must be unique within the data cube.
		@hibernate.property column="`filter_no`" type="short" not-null="true" 
			update="false" unique-key="DataCubeFilter_uniquefilter"
	*/
	public short getFilterNo()
	{
		return filterNo;
	}
	public void setFilterNo(short filterNo)
	{
		this.filterNo = filterNo;
	}
	
	private long bytes;
	/**
		The number of bytes the analysed data in this cube occupies on disk.
		@hibernate.property column="`bytes`" type="long" not-null="true" update="false"
	*/
	public long getBytes()
	{
		return bytes;
	}
	public void setBytes(long bytes)
	{
		this.bytes = bytes;
	}
}
