/*
  $Id: MimeTypeData.java 7015 2015-11-17 12:57:57Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;

/**
	This class holds information about mimetypes
	
	@author enell
	@version 2.0
	@see net.sf.basedb.core.MimeType
	@see <a href="../../../../../../html/developer/api/data_api.files.html">Developer documentation: Files and directories</a>
	@base.modified $Date: 2015-11-17 13:57:57 +0100 (Tue, 17 Nov 2015) $
	@hibernate.class table="`MimeTypes`" lazy="false"
*/
public class MimeTypeData
	extends BasicData
	implements NameableData, RemovableData, RegisteredData
{
	public MimeTypeData()
	{}

	public static final int MAX_EXTENSION_LENGTH = 255;
	
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	private String name;
	@Override
	public String getName()
	{
		return name;
	}
	@Override
	public void setName(String name)
	{
		this.name = name;
	}

	private String description;
	@Override
	public String getDescription()
	{
		return description;
	}

	@Override
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private Integer removedBy;
	@Override
	public Integer getRemovedBy()
	{
		return removedBy;
	}
	@Override
	public void setRemovedBy(Integer removedBy)
	{
		this.removedBy = removedBy;
	}
	// -------------------------------------------
	/*
		From the RegisteredData interface
		-------------------------------------------
	*/
	private Date entryDate;
	@Override
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -------------------------------------------

	private String extension;
	/**
		Get the extension for this <code>MimeType</code> item.
		@hibernate.property column="extension" type="string" length="255" unique="true" not-null="true"
	*/
	public String getExtension()
	{
		return extension;
	}
	public void setExtension(String extension)
	{
		this.extension = extension;
	}
	
	private ItemSubtypeData fileType;
	/**
		Get the associated {@link ItemSubtypeData} item.
		@hibernate.many-to-one column="`filesubtype_id`" not-null="false"
	*/
	public ItemSubtypeData getFileType()
	{
		return fileType;
	}
	public void setFileType(ItemSubtypeData fileType)
	{
		this.fileType = fileType;
	}

	private boolean autoCompress;
	/**
		If files of this MIME type should use auto-compression or not.
		@since 2.5
		@hibernate.property column="`auto_compress`" type="boolean" not-null="true"
	*/
	public boolean getAutoCompress()
	{
		return autoCompress;
	}
	public void setAutoCompress(boolean autoCompress)
	{
		this.autoCompress = autoCompress;
	}
	
}
