/**
	$Id $

	Copyright (C) 2016 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import java.util.List;

import org.hibernate.dialect.function.SQLFunctionTemplate;
import org.hibernate.engine.spi.SessionFactoryImplementor;
import org.hibernate.type.Type;

import net.sf.basedb.core.HibernateUtil;

/**
	Helper class for mixing native SQL with HQL queries. The typical use
	case (the only so far) is to execute a subquery in a place were 
	HQL doesn't support subqueries (in WITH clause).
	The function definition takes one parameter which is the native 
	SQL to execute. Use '[' and ']' to indicate open and close quotes
	in the native SQL dialect.
	
	@author nicklas
	@since 3.10
*/
public class NativeSQLFunction 
	extends SQLFunctionTemplate
{
	public NativeSQLFunction()
	{
		super(null, "?1");
	}

	@SuppressWarnings("rawtypes")
	@Override
	public String render(Type argumentType, List args, SessionFactoryImplementor factory)
	{
		String sql = super.render(argumentType, args, factory);
		
		char oq = HibernateUtil.getDialect().openQuote();
		char cq = HibernateUtil.getDialect().closeQuote();
		sql = sql.substring(1, sql.length()-1);
		sql = sql.replace('[', oq).replace(']', cq);
		return sql;
	}

}
