/**
	$Id: SimpleSignalSender.java 5399 2010-09-02 12:09:15Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.signal;

/**
	A signal sender implementation that sends a specific signal
	to all handlers that supports it.
	
	@author Nicklas
	@since 2.16
	@base.modified $Date: 2010-09-02 14:09:15 +0200 (Thu, 02 Sep 2010) $
*/
public class SimpleSignalSender
	implements SignalSender
{

	private final Signal signal;
	
	/**
		Create a new signal sender.
		@param signal The signal to send
	*/
	public SimpleSignalSender(Signal signal)
	{
		this.signal = signal;
	}

	/*
		From the SignalSender interface
		-------------------------------
	*/
	@Override
	public void sendTo(SignalHandler handler)
	{
		if (handler.supports(signal))
		{
			handler.handleSignal(signal);
		}
	}
	// -------------------------------------

	@Override
	public String toString()
	{
		return "SimpleSignalSender(" + signal + ")";
	}
}
