/**
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.export.spotdata;

/**
	Defines method for converting a string representation to
	an exportable assay or dynamic field that can be used with
	{@link AbstractBioAssaySetExporter} implementations. String
	representations are useful for plug-ins that need to store
	multiple fields related to different types of properties.
	For example, a converter may lookup experimental factors, 
	formulas or reporter properties based on the name.
	
	@author nicklas
	@since 2.15
*/
public interface ExportableFieldConverter
{
	/**
		Create a reporter dynamic field for the given name. The 
		interpretation of the name is implementation-specific.
		The returned field is typically added to the exporter with
		{@link AbstractBioAssaySetExporter#addReporterField(DynamicField)}.
		
		@param name The name to look for
		@param required TRUE if this field is required
		@return A dynamic field or null if no field is found, but
			if required = TRUE an exception is thrown instead
	*/
	public DynamicField getReporterField(String name, boolean required);
	
	/**
		Create a spot dynamic field for the given name. The 
		interpretation of the name is implementation-specific. 
		The returned field is typically added to the exporter with
		{@link AbstractBioAssaySetExporter#addSpotField(DynamicField)}.
		
		@param name The name to look for
		@param required TRUE if this field is required
		@return A dynamic field or null if no field is found, but
			if required = TRUE an exception is thrown instead
	*/
	public DynamicField getSpotField(String name, boolean required);
	
	/**
		Create an assay field for the given name. The 
		interpretation of the name is implementation-specific. 
		The returned field is typically added to the exporter with
		{@link AbstractBioAssaySetExporter#addAssayField(AssayField)}.
		
		@param name The name to look for
		@param required TRUE if this field is required
		@return A dynamic field or null if no field is found, but
			if required = TRUE an exception is thrown instead
	*/
	public AssayField getAssayField(String name, boolean required);
	
}
