/**
	$Id: RethrowErrorHandlerFactory.java 6444 2014-04-09 12:21:20Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

/**
	Extension to the logging error handler factory that create error handlers
	that re-throw the original exception. If the exception is an instance of
	RuntimeException it is re-thrown as is, otherwise it is wrapped inside
	a new RuntimeException. This factory is it's own error handler and the 
	implementation is thread-safe.
	
	@author Nicklas
	@since 2.17
	@base.modified $Date: 2014-04-09 14:21:20 +0200 (Wed, 09 Apr 2014) $
*/
public class RethrowErrorHandlerFactory
	extends LoggingErrorHandlerFactory
{
	
	private static final org.slf4j.Logger log = 
		org.slf4j.LoggerFactory.getLogger(RethrowErrorHandlerFactory.class);

	
	public RethrowErrorHandlerFactory()
	{}

	/*
		From the ErrorHandlerFactory interface
		--------------------------------------
	*/
	@Override
	public ErrorHandler<Action> getErrorHandler(InvokationContext<? extends Action> context)
	{
		return this;
	}
	// --------------------------------------

	/*
		From the ErrorHandler interface
		--------------------------------------
	*/
	@Override
	public void handleError(InvokationContext<? extends Action> context, Action action, 
		String message, Throwable t)
	{
		super.handleError(context, action, message, t);
		if (t instanceof RuntimeException)
		{
			throw (RuntimeException)t;
		}
		throw new RuntimeException(t);
	}
	// --------------------------------------
	
}
