/**
	$Id: IdentityFilter.java 4740 2009-02-05 13:39:25Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.filter;

/**
	A filter implementation that only accepts an object if it is
	identical (==) to a given object.
	
	@author Nicklas
	@version 2.10
	@base.modified $Date: 2009-02-05 14:39:25 +0100 (Thu, 05 Feb 2009) $
*/
public class IdentityFilter<T>
	implements Filter<T>
{
	private final T object;

	/**
		Create a new filter. 
		@param object The only object that should pass the filter
	*/
	public IdentityFilter(T object)
	{
		this.object = object;
	}

	/**
		@return TRUE if the object is the same instance as the object given in the constructor, 
			FALSE otherwise
	*/
	@Override
	public boolean evaluate(T object)
	{
		return this.object == object;
	}
	
}
