/**
	$Id: GenericIntensityParser.java 7665 2019-03-20 13:31:03Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.importer.spotdata;

import java.util.List;

import net.sf.basedb.util.basefile.BaseFileParser;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.FlatFileParser.Data;

/**
	Generic spot intensity parser implementation that 
	looks for 'intensity1', ..., 'intensityN' data columns for any
	number of channels.
	
	@author Nicklas
	@version 2.14
	@base.modified $Date: 2019-03-20 14:31:03 +0100 (Wed, 20 Mar 2019) $
*/
public class GenericIntensityParser
	implements SpotIntensityParser
{
	
	private final int channels;
	private int[] index;
	
	/**
		Creates a new parser for the specified number of channels.
	*/
	public GenericIntensityParser(int channels)
	{
		this.channels = channels;
	}
	
	/* 
		From the SpotIntensityParser interface
		--------------------------------------
	*/
	@Override
	public boolean hasRequiredAssayFields(BaseFileParser parser, FlatFileParser ffp,
			List<String> assayFields)
	{
		for (int i = 1; i <= channels; ++i)
		{
			int index = assayFields.indexOf(parser.getRedefinedColumnName("spots", "intensity" + i));
			if (index < 0) return false;
		}
		return true;
	}

	@Override
	public void beginSection(BaseFileParser parser, FlatFileParser ffp, 
			List<String> assayFields)
	{
		index = new int[channels];
		for (int i = 1; i <= channels; ++i)
		{
			index[i-1] = assayFields.indexOf(parser.getRedefinedColumnName("spots", "intensity" + i));
		}
	}

	@Override
	public boolean setIntensities(Data data, float[] intensities, int firstIndex)
	{
		for (int i = 0; i < channels; ++i)
		{
			Float value = data.getFloat(index[i] + firstIndex, null, true);
			if (value == null) return false;
			intensities[i] = value;
		}
		return true;
	}
	// ---------------------------------------

}
