/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Include;

/**
	Transformation context that can be used to keep track of vital
	information when doing multi-step transformations.

	@author Nicklas
	@since 3.5
*/
public class TransformContext 
{

	private final DbControl dc;
	private Collection<Include> include;
	
	/**
		Creates a new transformation context that will use the given DbControl
		for database access.
	*/
	public TransformContext(DbControl dc)
	{
		this.dc = dc;
		this.include = Include.ALL;
	}
	
	/**
		Get the DbControl to use in this transformation context.
	*/
	public DbControl getDbControl()
	{
		return dc;
	}
	
	/**
		Include options when executing queries to find
		parent/child items. Default is Include.ALL.
	*/
	public Collection<Include> getInclude()
	{
		return include;
	}
	
	public void setInclude(Collection<Include> include)
	{
		this.include = include;
	}
	
	/**
		Get a cache that can be used for storing transformation
		results for some time. The cache is stored in the current
		session and is lost if the user logs out.
		@since 3.16
	*/
	public TransformCache getCache()
	{
		TransformCache cache = dc.getSessionControl().getSessionSetting(TransformCache.class.getName());
		if (cache == null)
		{
			cache = new TransformCache(20);
			dc.getSessionControl().setSessionSetting(TransformCache.class.getName(), cache);
		}
		return cache;
	}

}
