/*
	$Id $

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/


import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioSource;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemList;
import net.sf.basedb.core.ItemProxy;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Listable;
import net.sf.basedb.core.Permission;

public class TestItemList
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing item lists");
		write_header();
		// Standard tests: create, load, list
		int id1 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist1", true);
		int id2 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist2", true);
		int id3 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist3", true);
		int id4 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist4", true);
		int id5 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist5", true);
		int id6 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist6", true);
		int id7 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist7", true);
		test_load(id1);
		test_list(-1);

		// Create items to use in the lists
		int bs1 = TestBioSource.test_create("bs.1", false);
		int bs2 = TestBioSource.test_create("bs.2", false);
		int bs3 = TestBioSource.test_create("bs.3", false);
		int bs4 = TestBioSource.test_create("bs.4", false);
		int bs5 = TestBioSource.test_create("bs.5", false);

		test_add_members(id1, bs1, bs2, bs3);
		test_list_members(id1, 3);
		test_add_members(id2, bs3, bs4, bs5); // One in common with 'id1'

		int bs6 = test_create_and_add_member(id7);
		test_list_members(id7, 1);
		
		int bs7 = test_union(id3, 6, id1, id2); // 'id3' is now the union of 'id1' and 'id2' + one new item
		test_list_members(id3, 6);
		
		int bs8 = test_intersect(id4, 3, id1, id2); // 'id4' is now the intersection of 'id1' and 'id2' + one new item
		test_list_members(id4, 3);
		
		test_remove(id5, id3, 4, id4); // 'id5' is now the complement of 'id3' to 'id4'
		test_retain(id6, id2, 3, id1); // 'id6' is now the intersection on 'id1' and 'id2'
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		// Remove an item and check that the size property is correct
		TestBioSource.test_delete(bs3);
		test_list_members(id1, 4);
		
		test_remove_members(id1, bs1, bs2, bs7, bs8);
		test_list_members(id1, 0);
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		// Standard test: Delete
		test_delete(id1);
		test_delete(id2);
		test_delete(id3);
		test_delete(id4);
		test_delete(id5);
		test_delete(id6);
		test_delete(id7);
		TestBioSource.test_delete(bs1);
		TestBioSource.test_delete(bs2);
		TestBioSource.test_delete(bs4);
		TestBioSource.test_delete(bs5);
		TestBioSource.test_delete(bs6);
		TestBioSource.test_delete(bs7);
		TestBioSource.test_delete(bs8);
		write("++Testing item lists "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(Item memberType, String externalId, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.ITEMLIST)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemList bl = ItemList.getNew(dc, memberType);
			bl.setExternalId(externalId);
			if (setAll)
			{
				bl.setName("Test item list");
				bl.setDescription("Added at "+new Date());
			}
			dc.saveItem(bl);
			dc.commit();
			id = bl.getId();
			write_item(0, bl);
			write("--Create item list OK");
		}
		catch (Throwable ex)
		{
			write("--Create item list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemList bl = ItemList.getById(dc, id);
			write_item(0, bl);
			write("--Load item list OK");
		}
		catch (Throwable ex)
		{
			write("--Load item list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemQuery<ItemList> q = ItemList.getQuery();
			ItemResultList<ItemList> l = q.list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List item lists OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List item lists FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemList bl = ItemList.getById(dc, id);
			dc.deleteItem(bl);
			Set<ItemProxy> using = bl.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(bl + " is used by " + using);
			}
			dc.commit();
			write("--Delete item list OK");
		}
		catch (Throwable ex)
		{
			write("--Delete item list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExternal ID\tSize");
			write("-- \t-- \t--------- \t-----------\t-----------\t----");
		}
	}
	static void write_item(int i, ItemList bl)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+bl.getId()+"\t"+bl.getName()+"\t"+
			bl.getDescription()+"\t"+bl.getExternalId()+"\t"+bl.getSize());
	}
	static void write_item(int i, Listable bm)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+bm.getId()+"\t"+bm.getName()+"\t"+
			bm.getType());
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_add_members(int id, int... itemIds)
	{
		if (id == 0 || itemIds == null || itemIds.length == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemList bl = ItemList.getById(dc, id);
			int oldSize = bl.getSize();
			for (int iid : itemIds)
			{
				Listable bm = (Listable)bl.getMemberType().getById(dc, iid);
				bl.add(bm);
			}
			dc.commit();
			
			dc = TestUtil.getDbControl();
			bl = ItemList.getById(dc, id);
			int newSize = bl.getSize();
			if (newSize != oldSize + itemIds.length)
			{
				throw new BaseException("new size=" + newSize + "; old size="+oldSize);
			}
			write("--Add items to list OK (new size=" + newSize + ")");
		}
		catch (Throwable ex)
		{
			write("--Add items to list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static int test_create_and_add_member(int id)
	{
		if (id == 0) return 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			BioSource bs = BioSource.getNew(dc);
			dc.saveItem(bs);
			
			ItemList bl = ItemList.getById(dc, id);
			bl.add(bs);
			
			dc.commit();
			id = bs.getId();
			write_item(0, bs);
			write("--Create and add member to list OK");
		}
		catch (Throwable ex)
		{
			write("--Create and add member to list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	
	static void test_remove_members(int id, int... items)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemList bl = ItemList.getById(dc, id);
			int oldSize = bl.getSize();
			for (int itemId : items)
			{
				Listable bm = (Listable)bl.getMemberType().getById(dc, itemId);
				bl.removeItem(bm);
			}
			dc.commit();
			dc = TestUtil.getDbControl();
			bl = ItemList.getById(dc, id);
			int newSize = bl.getSize();
			if (newSize != oldSize - items.length)
			{
				throw new BaseException("new size=" + newSize + "; old size="+oldSize);
			}
			write("--Remove items from list OK (new size=" + newSize + ")");
		}
		catch (Throwable ex)
		{
			write("--Remove items from list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list_members(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemList bl = ItemList.getById(dc, id);
			ItemQuery<? extends Listable> query = bl.getMembers();
			ItemResultList<? extends Listable> l = query.list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (bl.getSize() != l.size())
			{
				String msg = "List size is " + bl.getSize() + " but query returned " + l.size() + " members";
				if (expectedResults >= 0)
				{
					msg += " (expected " + expectedResults +")";
				}
				throw new BaseException(msg);
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List members in item list OK (" + l.size() + ")");
		}
		catch (Throwable ex)
		{
			write("--List members in item list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static int test_union(int toId, int expectedSize, int... fromId)
	{
		if (toId == 0) return 0;
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			
			BioSource bs = BioSource.getNew(dc);
			dc.saveItem(bs);
			
			ItemList to = ItemList.getById(dc, toId);
			List<ItemList> from = new ArrayList<ItemList>(fromId.length);
			for (int rlId : fromId)
			{
				ItemList list = ItemList.getById(dc, rlId);
				list.add(bs);
				from.add(list);
			}
			to.addUnion(from);
			dc.commit();
			id = bs.getId();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Create union OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Create union FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_intersect(int toId, int expectedSize, int... fromId)
	{
		if (toId == 0) return 0;
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			
			BioSource bs = BioSource.getNew(dc);
			dc.saveItem(bs);
			
			ItemList to = ItemList.getById(dc, toId);
			List<ItemList> from = new ArrayList<ItemList>(fromId.length);
			for (int rlId : fromId)
			{
				ItemList list = ItemList.getById(dc, rlId);
				list.add(bs);
				from.add(list);
			}
			to.addIntersection(from);
			dc.commit();
			id = bs.getId();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Create intersection OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Create intersection FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_remove(int toId, int baseId, int expectedSize, int... complementId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemList to = ItemList.getById(dc, toId);
			to.addAll(ItemList.getById(dc, baseId));
			List<ItemList> from = new ArrayList<ItemList>(complementId.length);
			for (int rlId : complementId)
			{
				from.add(ItemList.getById(dc, rlId));
			}
			to.removeUnion(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Remove items OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Remove items FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_retain(int toId, int baseId, int expectedSize, int... complementId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemList to = ItemList.getById(dc, toId);
			to.addAll(ItemList.getById(dc, baseId));
			List<ItemList> from = new ArrayList<ItemList>(complementId.length);
			for (int rlId : complementId)
			{
				from.add(ItemList.getById(dc, rlId));
			}
			to.retainUnion(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Retain items OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Retain items FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
}

