/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var AnnotationTypes = function()
{
	var annotationTypes = {};
	
	/**
		Initialize the page.
	*/
	annotationTypes.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', annotationTypes.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabValidator('settings.info', annotationTypes.validateAnnotationType);
			
			// Options
			Buttons.addClickHandler('interfaceBox', annotationTypes.interfaceOnClick);
			Buttons.addClickHandler('interfaceList', annotationTypes.interfaceOnClick);
			Buttons.addClickHandler('interfaceButtons', annotationTypes.interfaceOnClick);
			annotationTypes.interfaceOnClick();
			
			// Item type lists
			Buttons.addClickHandler('btnMoveLeft', Forms.moveSelectedOnClick);
			Buttons.addClickHandler('btnMoveRight', Forms.moveSelectedOnClick);
			Events.addEventHandler('enabled', 'dblclick', Forms.moveSelectedOnClick);
			Events.addEventHandler('disabled', 'dblclick', Forms.moveSelectedOnClick);
			
			// Units
			annotationTypes.initQuantities();
			Events.addEventHandler('quantity_id', 'change', annotationTypes.quantityOnChange);
			Buttons.addClickHandler('btnAddUsableUnits', annotationTypes.addToUsableUnits);
			Events.addEventHandler('allUnits', 'dblclick', annotationTypes.addToUsableUnits);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'ANNOTATIONTYPE', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'ANNOTATIONTYPE'};
			var tableAttributes = {'table-id': 'annotationTypes'};
			Buttons.addClickHandler('mnuNewAnnotationType', annotationTypes.newItem);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}

	annotationTypes.newItem = function(event)
	{
		var valueType = Data.get(event.target, 'value-type');
		Items.newItem('ANNOTATIONTYPE', '&value_type='+valueType);
	}

	annotationTypes.validateAnnotationType = function()
	{
		var frm = document.forms['annotationType'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}
	
	annotationTypes.save = function()
	{
		var frm = document.forms['annotationType'];
		if (TabControl.validateActiveTab('settings'))
		{
			var enabled = frm.enabled;
			for (var i = 0; i < enabled.length; i++)
			{
				enabled.options[i].selected = true;
			}
			Link.exportActions('categories');
			if (Doc.element('usableUnits'))
			{
				Link.exportActions('usableUnits');
			}
			frm.project_annotations.disabled = false;
			frm.submit();
		}
	}
	
	annotationTypes.interfaceOnClick = function()
	{
		var frm = document.forms['annotationType'];
		if (frm['interface'])
		{
			var isEnum = Forms.getCheckedRadio(frm['interface']).value != 'box';
			if (frm.minValue) frm.minValue.disabled = isEnum;
			if (frm.maxValue) frm.maxValue.disabled = isEnum;
			if (frm.width) frm.width.disabled = isEnum;
			if (frm.maxLength) frm.maxLength.disabled = isEnum;
			if (frm.values) frm.values.disabled = !isEnum;
		}
	}

	/**
		Initialize the 'Quantities'. If a current
		quantity is specified the 'Default unit' list is initialized
		as well.
	 */
	annotationTypes.initQuantities = function()
	{
		var frm = document.forms['annotationType'];
		var quantities = Data.json('page-data', 'units');
		var currentQuantityId = Data.int('page-data', 'current-quantity');
		
		for (var i = 0; i < quantities.length; i++)
		{
			var quantity = quantities[i];
			var option = new Option(quantity.name, quantity.id, false, quantity.id == currentQuantityId);
			option.quantity = quantity;
			frm.quantity_id[frm.quantity_id.length] = option;
		}
		
		// Initialize units for the current quantity
		if (currentQuantityId > 0) 
		{
			annotationTypes.quantityOnChange();
		}
	}
	
	/**
		If the selected quantity is changed, we need to update the list of
		default units and also the two 'usable units' lists.
	*/
	annotationTypes.quantityOnChange = function(event)
	{
		var frm = document.forms['annotationType'];
		var quantity = frm.quantity_id[frm.quantity_id.selectedIndex].quantity;
		var currentUnitId = Data.int('page-data', 'current-unit');
		
		// Update the default unit list with units from the selected quantity
		frm.unit_id.length = 0;
		frm.allUnits.length = 0;
		
		if (event)
		{
			// Clear the 'usable units' list
			Link.removeAll(frm.usableUnits);
		}
		
		if (quantity)
		{
			for (var i = 0; i < quantity.units.length; i++)
			{
				var unit = quantity.units[i];
				var option = new Option(unit.name + ' [ '+unit.symbol+' ]', unit.id, false, currentUnitId == unit.id);
				option.unit = unit;
				frm.unit_id[frm.unit_id.length] = option;
				
				var addToAllUnits = true;
				for (var j = 0; j < frm.usableUnits.length; j++)
				{
					if (frm.usableUnits[j].value == 'UNIT.'+unit.id)
					{
						addToAllUnits = false;
						j = frm.usableUnits.length;
					}
				}
				if (addToAllUnits)
				{
					frm.allUnits[frm.allUnits.length] = new Option(unit.name + ' [ '+unit.symbol+' ]', unit.id);
				}
			}
		}
	}
	
	/**
		Event handler for adding selected units from the 'All units' list to
		the 'Usable units' list.
	*/
	annotationTypes.addToUsableUnits = function(event)
	{
		var frm = document.forms['annotationType'];
		for (var i = 0; i < frm.allUnits.length; i++)
		{
			var option = frm.allUnits[i];
			if (option.selected)
			{
				Link.addItem(frm.usableUnits, 'UNIT', {'id': option.value, 'name': option.text });
				frm.allUnits[i--] = null;
			}
		}
	}	
	
	return annotationTypes;
}();

if (Doc.getPageId() == 'edit-page') 
{
	Doc.addElementInitializer(Forms.numberOnlyInitializer);
}
Doc.onLoad(AnnotationTypes.initPage);
