<%-- $Id: edit_mimetype.jsp 7604 2019-02-25 12:19:50Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.Application"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.MimeType"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.tabcontrol.TabAction"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final Item itemType = Item.MIMETYPE;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	String title = null;
	MimeType mimeType = null;
	boolean readCurrentFileType = true;
	int currentFileTypeId = 0;
	boolean autoCompress = false;

	if (itemId == 0)
	{
		title = "Create mime type";
		cc.removeObject("item");
		currentFileTypeId = Values.getInt(cc.getPropertyValue("fileType"), 0);
		if (currentFileTypeId == 0)
		{
			currentFileTypeId = Values.getInt(cc.getRecent(Item.ITEMSUBTYPE.name(), 0), 0);
		}
		autoCompress = Values.getBoolean(cc.getPropertyValue("autoCompress"), autoCompress);
	}
	else
	{
		mimeType = MimeType.getById(dc, itemId);
		mimeType.checkPermission(Permission.WRITE);
		cc.setObject("item", mimeType);
		title = "Edit MIME type -- " + HTML.encodeTags(mimeType.getName() + " (" + mimeType.getExtension() + ")");
		autoCompress = mimeType.getAutoCompress();
		try
		{
			ItemSubtype ft = mimeType.getFileType();
			if (ft != null) currentFileTypeId = ft.getId();
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentFileType = false;
		}
	}
	
	// Query to retrieve file types
	final ItemQuery<ItemSubtype> fileTypeQuery = ItemSubtype.getQuery(Item.FILE);
	fileTypeQuery.order(Orders.asc(Hql.property("name")));
	fileTypeQuery.setCacheResult(true);
	
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), mimeType);
	ExtensionsInvoker<TabAction> invoker = EditUtil.useEditExtensions(jspContext);
	%>
	<base:page type="popup" title="<%=title%>" id="edit-page">
	<base:head scripts="tabcontrol-2.js,~mimetypes.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%> <base:help tabcontrol="settings" /></h1>
		<form action="index.jsp?ID=<%=ID%>" method="post" name="mimeType">
		<input type="hidden" name="cmd" value="UpdateItem">

		<t:tabcontrol id="settings" 
			subclass="content dialogtabcontrol"
			position="bottom" remember="<%=mimeType != null%>"
			extensions="<%=invoker%>">
		<t:tab id="info" title="MIME type" helpid="mimetype.edit">
			<table class="fullform input100 smaller">
			<tr>
				<th>File extension</th>
				<td><input class="text required auto-init" data-auto-init="<%=mimeType == null ? "focus-select" : "focus" %>"
					type="text" name="extension" style="width: 10em;"
					value="<%=HTML.encodeTags(mimeType == null ? cc.getPropertyValue("extension") : mimeType.getExtension())%>" 
					maxlength="<%=MimeType.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>MIME type</th>
				<td><input class="text required" type="text" name="name" 
					value="<%=HTML.encodeTags(mimeType == null ? Values.getString(cc.getPropertyValue("name"), "New MIME type") : mimeType.getName())%>" 
					maxlength="<%=MimeType.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>File type</th>
				<td>
					<select name="filetype_id" <%=!readCurrentFileType ? "disabled readonly class=\"disabled\"" : "class=\"selectionlist\""%>>
					<%
					if (!readCurrentFileType)
					{
						%>
						<option value="-1">- denied -
						<%
					}
					else
					{
						%>
						<option value="0">- none -
						<%
						ItemResultList<ItemSubtype> fileTypes = fileTypeQuery.list(dc);
						for (ItemSubtype fileType : fileTypes)
						{
							int id = fileType.getId();
							%>
							<option 
								value="<%=mimeType != null && id == currentFileTypeId ? -id : id%>" 
								<%=id == currentFileTypeId ? "selected" : ""%>
								><%=HTML.encodeTags(fileType.getName())%>
							<%
						}
					}
					%>
					</select>
				</td>
				<td></td>
			</tr>
			<tr>
				<th>Auto compress</th>
				<td>
					<input type="radio" name="autoCompress" id="autoCompressNo" value="0" 
						<%=!autoCompress ? "checked" : ""%>><label for="autoCompressNo">no</label>
					<input type="radio" name="autoCompress" id="autoCompressYes" value="1" 
						<%=autoCompress ? "checked" : ""%>><label for="autoCompressYes">yes</label>
				<td></td>
			</tr>
			<%
			if (!Application.autoCompressionEnabled())
			{
				%>
				<tr>
					<th class="subprompt"></th>
					<td>
						<div class="messagecontainer help" style="margin: 0.5em;">
							Auto-compression has been disabled in <code>base.config</code>
						</div>
					</td>
					<td></td>
				</tr>	
				<%
			}
			%>
			<tr class="dynamic">
				<th>Description</th>
				<td>
					<textarea class="text" rows="6" name="description" id="description"
						><%=HTML.encodeTags(mimeType == null ? cc.getPropertyValue("description") : mimeType.getDescription())%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="description" title="Description" />
				</td>
			</tr>
			</table>
		</t:tab>
		</t:tabcontrol>
		</form>
		
		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
