<%-- $Id: edit_configuration.jsp 7605 2019-02-26 10:10:15Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.PluginConfiguration"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.tabcontrol.TabAction"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final Item itemType = Item.PLUGINCONFIGURATION;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final int cloneId = Values.getInt(request.getParameter("clone_id"));
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	String title = null;
	PluginConfiguration configuration = null;
	PluginDefinition currentPlugin = null;
	PluginConfiguration originalConfiguration = null;
	PluginDefinition originalCurrentPlugin = null;
	boolean readCurrentPlugin = true;
	String name = null;
	String description = null;
	
	// Load recently used items
	List<PluginDefinition> recentPlugins = cc.getRecent(dc, Item.PLUGINDEFINITION);

	if (itemId == 0 && cloneId == 0)
	{
		title = "Create configuration";
		cc.removeObject("item");
		int pluginId = Values.getInt(request.getParameter("plugindefinition_id"));
		if (pluginId != 0)
		{
			currentPlugin = PluginDefinition.getById(dc, pluginId);
		}
		else if (cc.getPropertyFilter("pluginDefinition.name") != null)
		{
			currentPlugin = Base.getFirstMatching(dc, PluginDefinition.getQuery(), "name", cc.getPropertyFilter("pluginDefinition.name"));
		}
		if (currentPlugin != null)
		{
			name = "New " + currentPlugin.getName() + " configuration";
		}
		else
		{
			name = Values.getString(cc.getPropertyValue("name"), "New configuration");
		}
		description = cc.getPropertyValue("description");
	}
	else if (cloneId != 0)
	{
		originalConfiguration = PluginConfiguration.getById(dc, cloneId);
		cc.removeObject("item");
		title = "Create configuration";
		name = "Copy of " + originalConfiguration.getName();
		description = originalConfiguration.getDescription();
		try
		{
			currentPlugin = originalConfiguration.getPluginDefinition();
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentPlugin = false;
		}
	}
	else
	{
		configuration = PluginConfiguration.getById(dc, itemId);
		configuration.checkPermission(Permission.WRITE);
		cc.setObject("item", configuration);
		title = "Edit configuration -- " + HTML.encodeTags(configuration.getName());
		name = configuration.getName();
		description = configuration.getDescription();
		try
		{
			currentPlugin = configuration.getPluginDefinition();
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentPlugin = false;
		}
	}
	
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), configuration);
	ExtensionsInvoker<TabAction> invoker = EditUtil.useEditExtensions(jspContext);
	%>
	<base:page type="popup" title="<%=title%>" id="edit-page">
	<base:head scripts="tabcontrol-2.js,~configurations.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%> <base:help tabcontrol="settings" /></h1>
		<form action="index.jsp?ID=<%=ID%>" method="post" name="configuration">
		<input type="hidden" name="cmd" value="UpdateItem">
		<input type="hidden" name="configure" value="0">
		<input type="hidden" name="copyfrom" value="<%=originalConfiguration != null ? originalConfiguration.getId() : 0%>">

		<t:tabcontrol id="settings"
			subclass="content dialogtabcontrol"
			position="bottom" remember="<%=configuration != null%>" 
			extensions="<%=invoker%>">
		<t:tab id="info" title="Configuration" helpid="pluginconfiguration.edit">
			<table class="fullform input100 smaller">
			<tr>
				<th>Name</th>
				<td><input class="text required auto-init" data-auto-init="<%=configuration == null ? "focus-select" : "focus" %>"
					type="text" name="name" 
					value="<%=HTML.encodeTags(name)%>" 
					maxlength="<%=PluginConfiguration.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>Plugin</th>
				<td>
					<%
					if (configuration == null)
					{
						String pluginMainType = Values.getStringOrNull(request.getParameter("plugin.mainType"));
						String pluginItemType = Values.getStringOrNull(request.getParameter("plugin.itemType"));
						
						%>
						<base:select 
							id="plugindefinition_id"
							clazz="selectionlist required"
							required="true"
							current="<%=currentPlugin%>"
							recent="<%=recentPlugins%>"
							newitem="true"
							data-plugin-main-type="<%=pluginMainType == null ? "" : Plugin.MainType.valueOf(pluginMainType).getValue()%>"
							data-plugin-item-type="<%=pluginItemType == null ? "": Item.valueOf(pluginItemType).getValue()%>"
						/>
						<%
					}
					else
					{
						%>
						<%=currentPlugin == null ? "<i>- denied -</i>" : HTML.encodeTags(currentPlugin.getName())%>
						<%
					}
					%>
				</td>
				<td></td>
			</tr>
			<tr class="dynamic">
				<th>Description</th>
				<td>
					<textarea class="text" rows="6" name="description" id="description"
						><%=HTML.encodeTags(description)%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="description" title="Description" />
				</td>
			</tr>
			</table>
		</t:tab>
		
		<t:tab id="annotations" title="Annotations" 
			helpid="annotations.edit" tooltip="Enter values for annotations"
			visible="<%=currentPlugin != null && currentPlugin.supports("net.sf.basedb.core.plugin.AnnotationSetterPlugin")%>">
			<jsp:include page="../../common/annotations/annotate_frameset.jsp">
				<jsp:param name="item_type" value="<%=itemType.name()%>" />
				<jsp:param name="item_id" value="<%=itemId%>" />
				<jsp:param name="ID" value="<%=ID%>" />
			</jsp:include>
		</t:tab>
		</t:tabcontrol>
		</form>
		
		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="btnSaveAndConfigure" 
				image="runplugin.png" title="Save and configure" 
				visible="<%=currentPlugin == null || currentPlugin.supportsConfigurations()%>" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
