<%-- $Id: index.jsp 7703 2019-04-11 07:31:26Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.ArrayBatch"
	import="net.sf.basedb.core.ArrayDesign"
	import="net.sf.basedb.core.Protocol"
	import="net.sf.basedb.core.Hardware"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.MultiPermissions"
	import="net.sf.basedb.core.OwnedItem"
	import="net.sf.basedb.core.AnnotatedItem"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.ItemAlreadyExistsException"
	import="net.sf.basedb.util.RemovableUtil"
	import="net.sf.basedb.util.ShareableUtil"
	import="net.sf.basedb.util.OwnableUtil"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.clients.web.util.HTML"
	import="java.util.Enumeration"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.edit.OnSaveAction"
	import="net.sf.basedb.clients.web.extensions.edit.OnSaveRenderer"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnExportRenderer"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnAction"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnUtil"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.List"
	import="java.util.ArrayList"
	import="java.util.Collections"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%!
	private static final ItemContext defaultContext = Base.createDefaultContext("name", "name,arrayDesign,slides,description");
	private static final Item itemType = Item.ARRAYBATCH;
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String cmd = request.getParameter("cmd");
final String root = request.getContextPath()+"/";
final String mode = request.getParameter("mode");
final String callback = request.getParameter("callback");
final String itemId = request.getParameter("item_id");
final String listPage = "list_batches.jsp?ID="+ID
	+(mode == null ? "" : "&mode="+mode)
	+(callback == null ? "" : "&callback="+callback)
	+(itemId == null ? "" : "&item_id="+itemId);
final String viewPage = "view_batch.jsp?ID="+ID;
final String editPage = "edit_batch.jsp?ID="+ID;

String forward = null;
String redirect = null;
String message = null;
DbControl dc = null;

try
{
	if (cmd == null || "List".equals(cmd))
	{
		// Display the list page without updatinging the current context
		Base.getAndSetCurrentContext(sc, itemType, null, defaultContext, true);
		redirect = listPage;
	}
	else if ("UpdateContext".equals(cmd))
	{
		// Display the list page after updating the current context from the request parameters
		Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = listPage;
	}
	else if ("LoadContext".equals(cmd))
	{
		// Display the list page after loading a saved context
		int contextId = Values.getInt(request.getParameter("context"));
		Base.loadContext(sc, contextId, defaultContext);
		redirect = listPage;
	}

	else if ("ViewItem".equals(cmd))
	{
		// Display the view page for a single item 
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		forward = viewPage;
	}
	else if ("EditItem".equals(cmd))
	{
		// Display the edit page for a single item (should be opened in a popup)
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = editPage;
	}
	else if ("NewItem".equals(cmd))
	{
		// Display the edit page for a new item (should be opened in a popup)
		if (!sc.hasPermission(Permission.CREATE, itemType))
		{
			throw new PermissionDeniedException(Permission.CREATE, itemType.toString());
		}
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		cc.setId(0);
		forward = editPage;
	}
	else if ("UpdateItem".equals(cmd))
	{
		// Update the properties on an item (will close the popup)
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, defaultContext);
		final int maxRecent = Base.getMaxRecent(sc);
		dc = sc.newDbControl();
		ArrayBatch batch = cc.getObject("item");
		if (batch == null)
		{
			int designId = Values.getInt(request.getParameter("arraydesign_id"), -1);
			ArrayDesign design = ArrayDesign.getById(dc, designId);
			batch = ArrayBatch.getNew(dc, design);
			message = "Array batch created";
			dc.saveItem(batch);
			cc.setRecent(Item.ARRAYDESIGN, design, maxRecent);
		}
		else
		{
			dc.reattachItem(batch, false);
			message = "Array batch updated";
		}
		JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(Item.ARRAYBATCH), batch);
		ExtensionsInvoker<OnSaveAction> invoker = EditUtil.useOnSaveExtensions(jspContext);
		try
		{
			batch.setName(Values.getStringOrNull(request.getParameter("name")));
			batch.setDescription(Values.getStringOrNull(request.getParameter("description")));
			int protocolId = Values.getInt(request.getParameter("protocol_id"), -1);
			if (protocolId >= 0) // < 0 = denied or unchanged
			{
				Protocol pt = protocolId == 0 ? null : Protocol.getById(dc, protocolId);
				batch.setProtocol(pt);
				cc.setRecent(Item.PROTOCOL, pt, maxRecent);
			}
			int robotId = Values.getInt(request.getParameter("printrobot_id"), -1);
			if (robotId >= 0) // < 0 = denied or unchanged
			{
				Hardware hw = robotId == 0 ? null : Hardware.getById(dc, robotId);
				batch.setPrintRobot(hw);
				cc.setRecent(Item.HARDWARE, hw, maxRecent);
			}
	
			// Annotations tab
			Base.updateAnnotations(dc, batch, batch, request);
			
			// OnSave extensions
			invoker.render(OnSaveRenderer.ON_SAVE);
			dc.commit();
			invoker.render(OnSaveRenderer.ON_COMMIT);
		}
		catch (Exception ex)
		{
			invoker.render(OnSaveRenderer.onRollback(ex));
			throw ex;
		}
		finally
		{
			cc.removeObject("item");
		}
	}
	else if ("DeleteItem".equals(cmd))
	{
		// Delete a single item and then return to the view page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		RemovableUtil.setRemoved(dc, itemType, Collections.singleton(cc.getId()), true);
		dc.commit();
		redirect = viewPage;
	}
	else if ("DeleteItems".equals(cmd))
	{
		// Delete all selected items on the list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		int numTotal = cc.getSelected().size();
		int numRemoved = RemovableUtil.setRemoved(dc, itemType, cc.getSelected(), true);
		dc.commit();
		if (numTotal != numRemoved)
		{
			message = (numRemoved == 0 ? "No" : "Only "+numRemoved+" of "+numTotal) + " items could be deleted, because you have no DELETE permission";
		}
		redirect = listPage;
	}
	else if ("RestoreItem".equals(cmd))
	{
		// Restore a single item and then return to the view page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		RemovableUtil.setRemoved(dc, itemType, Collections.singleton(cc.getId()), false);
		dc.commit();
		redirect = viewPage;
	}
	else if ("RestoreItems".equals(cmd))
	{
		// Restore all selected items on the list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		int numTotal = cc.getSelected().size();
		int numRemoved = RemovableUtil.setRemoved(dc, itemType, cc.getSelected(), false);
		dc.commit();
		if (numTotal != numRemoved)
		{
			message = (numRemoved == 0 ? "No" : "Only "+numRemoved+" of "+numTotal) + " items could be restored, because you have no WRITE permission";
		}
		redirect = listPage;
	}
	else if ("ShareItem".equals(cmd))
	{
		// Display a popup window for sharing a single item
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		MultiPermissions permissions = ShareableUtil.getMultiPermissions(dc, itemType, Collections.singleton(cc.getId()));
		dc.close();
		cc.setObject("MultiPermissions", permissions);
		redirect = "../../common/share/share.jsp?ID="+ID+"&item_type="+itemType.name();
	}
	else if ("ShareItems".equals(cmd))
	{
		// Display a popup window for sharing all selected items on the list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		MultiPermissions permissions = ShareableUtil.getMultiPermissions(dc, itemType, cc.getSelected());
		dc.close();
		cc.setObject("MultiPermissions", permissions);
		redirect = "../../common/share/share.jsp?ID="+ID+"&item_type="+itemType.name();
	}
	else if ("SetOwnerOfItem".equals(cmd))
	{
		// Change owner of items selected on a list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		OwnedItem item = (OwnedItem)itemType.getById(dc, cc.getId());
		cc.setObject("OwnedItems", Collections.singleton(item));
		redirect = "../../common/ownership/ownership.jsp?ID="+ID+"&item_type="+itemType.name();
	}
	else if ("SetOwnerOfItems".equals(cmd))
	{
		// Change owner of items selected on a list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		Set<OwnedItem> items = new HashSet<OwnedItem>();
		for (Integer id : cc.getSelected())
		{
			if (id != null) items.add((OwnedItem)itemType.getById(dc, id));
		}
		dc.close();
		cc.setObject("OwnedItems", items);
		redirect = "../../common/ownership/ownership.jsp?ID="+ID+"&item_type="+itemType.name();
	}
	else if ("InheritAnnotations".equals(cmd))
	{
		// Change owner of items selected on a list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		Set<AnnotatedItem> items = new HashSet<AnnotatedItem>();
		for (Integer id : cc.getSelected())
		{
			if (id != null) items.add((AnnotatedItem)itemType.getById(dc, id));
		}
		dc.close();
		cc.setObject("AnnotatedItems", items);
		redirect = "../../common/annotations/batch_inherit.jsp?ID="+ID+"&item_type="+itemType.name();
	}
	else if ("ExportItems".equals(cmd))
	{
		// Run an export plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		final ItemQuery<ArrayBatch> query = ArrayBatch.getQuery();
		dc = sc.newDbControl();
		cc.configureQuery(dc, query, true);
		cc.setQuery(query);
		JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.list(itemType), null);
		ExtensionsInvoker<ListColumnAction<ArrayBatch,?>> listInvoker = ListColumnUtil.useExtensions(jspContext);
		listInvoker.render(new ListColumnExportRenderer<ArrayBatch>(cc));
		dc.close();
		redirect = "../../common/export/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=LIST&title=Export+array+batches";
	}
	else if ("ExportItem".equals(cmd))
	{
		// Run an export plugin in single-item context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = "../../common/export/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=ITEM&title=Export+array+batch";
	}
	else if ("ImportItems".equals(cmd))
	{
		// Run an import plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		final ItemQuery<ArrayBatch> query = ArrayBatch.getQuery();
		dc = sc.newDbControl();
		cc.configureQuery(dc, query, true);
		dc.close();
		cc.setQuery(query);
		redirect = "../../common/import/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=LIST&title=Import+array+batches";
	}
	else if ("ImportItem".equals(cmd))
	{
		// Run an import plugin in single-item context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = "../../common/import/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=ITEM&title=Import+array+batch";
	}
	else if ("RunListPlugin".equals(cmd))
	{
		// Run another plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		final ItemQuery<ArrayBatch> query = ArrayBatch.getQuery();
		dc = sc.newDbControl();
		cc.configureQuery(dc, query, true);
		dc.close();
		cc.setQuery(query);
		redirect = "../../common/plugin/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=LIST&main_type=OTHER&title=Run+plugin";
	}
	else if ("RunPlugin".equals(cmd))
	{
		// Run another plugin in single-item context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = "../../common/plugin/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=ITEM&main_type=OTHER&title=Run+plugin";
	}
	else
	{
		throw new WebException("popup", "Invalid command", "The command {1} is not recognised as a valid command.", cmd);
	}
}
finally
{
	if (dc != null) dc.close();
}

if (forward != null)
{
	sc.setSessionSetting("alert-message", message);
	pageContext.forward(forward);
}
else if (redirect != null)
{
	sc.setSessionSetting("alert-message", message);
	response.sendRedirect(redirect);
}
else if (message == null)
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&wait=0");
}
else
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&message="+HTML.urlEncode(message));
}
%>

