<%-- $Id: jep_filter.jsp 7605 2019-02-26 10:10:15Z nicklas $
	------------------------------------------------------------------
	BioArray Software Environment (BASE) - http:// base.thep.lu.se/

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Job"
	import="net.sf.basedb.core.BioAssaySet"
	import="net.sf.basedb.core.BioAssay"
	import="net.sf.basedb.core.RawDataType"
	import="net.sf.basedb.core.RawDataProperty"
	import="net.sf.basedb.core.Experiment"
	import="net.sf.basedb.core.Formula"
	import="net.sf.basedb.core.Type"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.util.json.JsonUtil"
	import="net.sf.basedb.util.json.NameableConverter"
	import="org.json.simple.JSONObject"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%!
private static void addFormulaOption(StringBuilder options, String formula, String title, String description)
{
	options.append("<option value=\"").append(HTML.encodeTags(formula)).append("\"");
	options.append(" title=\"").append(HTML.encodeTags(description)).append("\"");
	options.append(">").append(HTML.encodeTags(title)).append("\n");
}
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
ItemContext cc = sc.getCurrentContext(Item.BIOASSAYSET);
DbControl dc = null;

try
{
	String errorMessage = sc.getSessionSetting("plugin.configure.errors.message");
	List<Throwable> errors = sc.getSessionSetting("plugin.configure.errors.list");
	
	dc = sc.newDbControl();
	Job job = sc.getSessionSetting("plugin.configure.job");
	dc.reattachItem(job, false);
	BioAssaySet source = job.getParameterValue("source");
	if (source == null && cc.getId() != 0) source = BioAssaySet.getById(dc, cc.getId());
	if (source == null) throw new WebException("popup", "No current bioassay set", 
		"Could not find any current bioassay set. Please select a bioassay set before trying again");
	source = BioAssaySet.getById(dc, source.getId());
	RawDataType rdt = source.getRawDataType();
	long numBioAssays = source.getBioAssays().count(dc);
	int maxRawMappings = source.getMaxRawMappingsForSpot();
	
	// Find selected bioassays
	List<BioAssay> bioAssays = job.getParameterValues("bioAssays");
	if (bioAssays == null)
	{
		ItemContext bioAssayContext = sc.getCurrentContext(Item.BIOASSAY);
		ItemQuery<BioAssay> bioAssayQuery = null;
		if (bioAssayContext.getSelected().size() > 0)
		{
			bioAssayQuery = source.getBioAssays();
			bioAssayQuery.restrict(
				Restrictions.in(Hql.property("id"), Expressions.parameter("selected"))
			);
			bioAssayQuery.setParameter("selected", bioAssayContext.getSelected(), Type.INT);
			bioAssays = bioAssayQuery.list(dc);
		}
	}
	
	JSONObject jsonBioAssays = new JSONObject();
	jsonBioAssays.put("itemType", "BIOASSAY");
	if (bioAssays != null)
	{
		jsonBioAssays.put("items", JsonUtil.toArray(bioAssays, new NameableConverter<BioAssay>()));
	}

	// Current parameter values
	String childName = Values.getString(job.getParameterValue("childName"), 
		source.getName().startsWith("Filtered") ? "" : "Filtered " + source.getName());
	String childDescription = job.getParameterValue("childDescription");
	String expression = job.getParameterValue("expression");
	Integer includeLimit = job.getParameterValue("includeLimit");
	Integer excludeLimit = job.getParameterValue("excludeLimit");
	
	// Predefined formulas
	ItemQuery<Formula> formulaQuery = Formula.getQuery(Formula.Type.COLUMN_RESTRICTION, rdt,
			source.getIntensityTransform());
	formulaQuery.order(Orders.asc(Hql.property("name")));
	formulaQuery.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
	ItemResultList<Formula> formulas = formulaQuery.list(dc);
	StringBuilder formulaOptions = new StringBuilder();
	for (Formula f : formulas)
	{
		String formula = f.getFormula(0);
		if (maxRawMappings == 1 || !formula.contains("raw("))
		{
			addFormulaOption(formulaOptions, formula, f.getName(), f.getDescription());
		}
	}
	%>
	<base:page type="popup" title="Filter bioassay set">
	<base:head scripts="linkitems-2.js,/plugins/net/sf/basedb/plugins/jep_plugins.js,/plugins/net/sf/basedb/plugins/jep_filter.js" />
	<base:body>
		<h1>Filter bioassay set <base:help helpid="jepfilter.options"/></h1>
		<div id="page-data" class="datacontainer"
			data-max-raw-mapping="<%=maxRawMappings %>"
			data-num-bioassays="<%=numBioAssays %>"
		></div>
		
		<form name="filter" action="index.jsp" method="post">
		<input type="hidden" name="ID" value="<%=ID%>">
		<input type="hidden" name="requestId" value="<%=request.getParameter("requestId")%>">
		<input type="hidden" name="cmd" value="SetParameters">
		<input type="hidden" name="parameter:source" value="<%=source.getId()%>">
		
		<div class="content bottomborder">
		<table class="fullform input100">
		<%
		if (errorMessage != null || (errors != null && errors.size() > 0))
		{
			%>
			<tr>
				<th></th>
				<td colspan="2">
				<div id="errors" style="margin-bottom: 12px;">
					<div class="messagecontainer error" style="margin: 0px;">
					<%=errorMessage %>
					<%
					if (errors != null && errors.size() > 0)
					{
						%>
						<div id="showerrorlist">
							<base:icon 
								id="btnShowError"
								image="gonext.png" 
								style="color: #FFFFFF;"
								tooltip="Show more information about the error"
							/>
						</div>
						<div id="errorlist" style="display: none; margin: 0px;">
							<base:icon 
								id="btnHideError"
								image="move_down.png" 
								style="color: #FFFFFF;" 
								tooltip="Show less information"
							/>
							<ol>
							<%
							for (Throwable t : errors)
							{
								%>
								<li><%=t.getMessage()%>
								<%
							}
							%>
							</ol>
						</div>
						<%
					}
					%>
				</div>
				</td>
			</tr>
			<%
		}
		%>
		<tr>
			<th>Source</th>
			<td colspan="2"><%=HTML.encodeTags(source.getName())%></td>
		</tr>
		<tr class="big">
			<th class="subprompt">Bioassays</th>
			<td>
				<input type="checkbox" name="allBioAssays" id="allBioAssays"
					><label for="allBioAssays">All bioassays</label><br>
				<div class="selectionlist">
					<table>
					<tr>
					<td>
						<select name="bioAssays" id="bioAssays" 
							class="auto-init"
							data-auto-init="link-container"
							data-initial-items="[<%=HTML.encodeTags(jsonBioAssays.toJSONString()) %>]"
							size="5" multiple>
						</select>
					</td>
					<td style="vertical-align: top;">
						<base:buttongroup vertical="true">
							<base:button 
								id="btnAddBioAssays"
								subclass="leftaligned"
								title="Add bioassays&hellip;" 
								tooltip="Add bioassays"
							/>
							<base:button 
								id="btnRemoveBioAssays"
								subclass="leftaligned auto-init"
								data-auto-init="remove-link"
								data-list-id="bioAssays"
								title="Remove" 
								tooltip="Remove the selected bioassays"
							/>
						</base:buttongroup>
					</td>
					</tr>
					</table>
				</div>
			</td>
			<td></td>
		</tr>
		<tr>
			<th>Child bioassay set</th>
			<td colspan="2"></td>
		</tr>
		<tr>
			<th class="subprompt">Name</th>
			<td>
				<input type="text" class="text required" maxlength="255" 
					name="parameter:childName" 
					value="<%=HTML.encodeTags(childName)%>"></td>
			<td></td>
		</tr>
		<tr class="dynamic">
			<th class="subprompt">Description</th>
			<td>
				<textarea class="text" rows="3" name="parameter:childDescription" id="childDescription"
					><%=HTML.encodeTags(childDescription) %></textarea>
			</td>
			<td style="width: 20px;">
				<base:zoom textarea="childDescription" title="Description" />
			</td>
		</tr>

		<tr>
			<th>Filter</th>
			<td>
				<select name="presets" id="presets" style="width: 30em;"
					data-formula-id="parameterExpression">
					<option value="">- select from list or enter formula below -
					<%=formulaOptions.toString()%>
				</select>
			</td>
			<td></td>
		</tr>
		<tr class="big">
			<th class="subprompt">Expression</th>
			<td>
				<textarea class="text required" name="parameter:expression"
					id="parameterExpression" rows="3" ><%=HTML.encodeTags(expression)%></textarea>
			</td>
			<td style="width: 20px;">
				<base:icon 
					id="btnExpressionBuilder" 
					data-formula-id="parameterExpression"
					data-title="Expression"
					data-formula-type="COLUMN_RESTRICTION"
					data-raw-data-type="<%=rdt.getId() %>"
					data-channels="<%=rdt.getChannels() %>"
					data-bioassayset="<%=source.getId() %>"
					image="expression_builder.png"
					tooltip="A simple utility for building expressions"
				/>
			</td>
		</tr>
		<tr>
			<th class="subprompt">Keep</th>
			<td>Keep non-matching spots from all bioassays if the filter matches that spot in at least<br>
				<input class="text" type="text" name="parameter:includeLimit" id="includeLimit" style="width: 5em;"
					maxlength="10" value="<%=includeLimit == null ? "" : includeLimit.toString()%>"
					>
				bioassay(s) [1 - <%=numBioAssays%>]
			</td>
			<td></td>
		</tr>
		<tr>
			<th class="subprompt">Exclude</th>
			<td>Exclude matching spots from all bioassays if the filter doesn't match that spot in at least<br>
				<input class="text" type="text" name="parameter:excludeLimit" id="excludeLimit" style="width: 5em;"
					maxlength="10" value="<%=excludeLimit == null ? "" : excludeLimit.toString()%>"
					>
				bioassay(s) [1 - <%=numBioAssays%>]
			</td>
			<td></td>
		</tr>
		</table>
		</div>
		</form>
	
		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnNext" title="Next" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>

