/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Plotter = function()
{
	var plotter = {};
	
	/**
		Initialize the page.
	*/
	plotter.initPage = function()
	{
		Buttons.addClickHandler('close', App.closeWindow);
		
		// Plot size
		Events.addEventHandler('width', 'keypress', Events.integerOnly);
		Events.addEventHandler('height', 'keypress', Events.integerOnly);
		
		// Y-axis
		Events.addEventHandler('yPresets', 'change', plotter.presetOnChange);
		Buttons.addClickHandler('btnExpressionBuilderY', plotter.openExpressionBuilder);
		
		// Plot type
		Events.addEventHandler('plotTypeAssay', 'change', plotter.plotTypeOnChange);
		Events.addEventHandler('plotTypeAnnotation', 'change', plotter.plotTypeOnChange);
		plotter.plotTypeOnChange();
		
		// Buttons
		Buttons.addClickHandler('btnPreviewPlot', plotter.previewPlot);
		Buttons.addClickHandler('btnViewPlot', plotter.viewPlot);
		Buttons.addClickHandler('btnDownloadPlot', plotter.downloadPlot);
		Buttons.addClickHandler('btnSavePlot', plotter.savePlotAs);
	}

	plotter.getPlotType = function()
	{
		var frm = document.forms['plot'];
		return Forms.getCheckedRadio(frm.plotType).value;
	}

	plotter.validateParameters = function()
	{
		var frm = document.forms['plot'];
		if (Strings.trim(frm.yFormula.value) == '')
		{
			Forms.showNotification(frm.yFormula, 'You must enter an expression for the Y axis');
			return false;
		}
		return true;
	}
	
	plotter.presetOnChange = function(event)
	{
		var list = event.currentTarget;
		var selected = list[list.selectedIndex];
		
		var labelField = Doc.element(Data.get(list, 'label-id'));
		var formulaField = Doc.element(Data.get(list, 'formula-id'));

		var frm = list.form;
		if (frm.averageMethod)
		{
			var avgMethod = Data.get(selected, 'average-method');
			Forms.selectListOption(frm.averageMethod, avgMethod);
		}
		
		formulaField.value = selected.value;
		if (labelField && selected.value != '') labelField.value = selected.text;
		list.selectedIndex = 0;
	}

	
	plotter.plotTypeOnChange = function()
	{
		var plotType = plotter.getPlotType();
		var isAssayPlot = plotType == 'assay';
		var frm = document.forms['plot'];
		frm.annotationTypeId.disabled = isAssayPlot;
		frm.subtype.disabled = !isAssayPlot;
	}

	plotter.openExpressionBuilder = function(event)
	{
		var formulaField = Doc.element(Data.get(event.currentTarget, 'formula-id'));
		
		if (!formulaField.disabled)
		{
			var title = Data.get(event.currentTarget, 'title');
			var formulaType = Data.get(event.currentTarget, 'formula-type');
			var rawDataType = Data.get(event.currentTarget, 'raw-data-type');
			var channels = Data.int(event.currentTarget, 'channels');
			var bioAssaySetId = Data.int(event.currentTarget, 'bioassayset');
			
			Dialogs.openExpressionBuilder(formulaField, title, formulaType, rawDataType, channels, bioAssaySetId);0
		}
	}

	
	plotter.generatePlotUrl = function(fullSize)
	{
		if (!plotter.validateParameters()) return null;

		var frm = document.forms['plot'];
		var url = App.getRoot() + 'views/experiments/explorer/plot?ID='+App.getSessionId();
		url += '&bioAssaySetId='+Data.get('page-data', 'bioassayset');
		url += '&reporterIndex='+Data.get('page-data', 'reporter-index');
		url += '&positionIndex='+Data.get('page-data', 'position-index');
		url += '&title='+encodeURIComponent(frm.title.value);
		url += '&subTitle='+encodeURIComponent(frm.subTitle.value);
		if (fullSize)
		{
			url += '&width='+frm.width.value;
			url += '&height='+frm.height.value;
		}
		
		url += '&y='+encodeURIComponent(frm.yFormula.value);
		url += '&yLog='+(frm.yLog.checked ? 1 : 0);
		url += '&yLabel='+encodeURIComponent(frm.yLabel.value);
		
		var plotType = plotter.getPlotType();
		url += '&type='+plotType;
		url += '&showXLabels='+(frm.hideXLabels.checked ? 0 : 1);
		
		if (plotType == 'assay')
		{
			url += '&subtype=' + frm.subtype.value;
			if (frm.averageMethod)
			{
				url += '&averageMethod=' + frm.averageMethod.value;
			}
		}
		else if (plotType == 'annotation')
		{
			url += '&annotationTypeId=' + frm.annotationTypeId.value;
		}
		url += '&' + new Date().getTime();
		return url;
	}

	plotter.previewPlot = function()
	{
		var url = plotter.generatePlotUrl(false);
		if (url)
		{
			url += '&width=540&height=360';

			var overlayImg = Doc.element('overlay');
			overlayImg.src = App.getRoot()+'images/plot_generating.gif';
			
			var tmpPlot = new Image();
			Events.addEventHandler(tmpPlot, 'load', plotter.previewLoaded);
			tmpPlot.src = url;
		}
	}
	
	plotter.previewLoaded = function(event)
	{
		var tmpPlot = event.currentTarget;
		
		var previewImg = Doc.element('preview');
		previewImg.src = tmpPlot.src;

		var overlayImg = Doc.element('overlay');
		overlayImg.src = App.getRoot() + 'images/blankbutton.gif';
	}

	plotter.viewPlot = function()
	{
		var plotUrl = plotter.generatePlotUrl(true);
		if (plotUrl)
		{
			var frm = document.forms['plot'];
			var width = parseInt(frm.width.value);
			var height = parseInt(frm.height.value);
			if (!width || width < 600) width = 600;
			if (!height || height < 400) height = 400;
			var url = '../../plotter/view.jsp?ID='+App.getSessionId();
			url += '&title='+encodeURIComponent(frm.title.value);
			url += '&plot='+encodeURIComponent(plotUrl);
			Dialogs.openPopup(url, 'ViewPlot', width+150, height+100);
		}
	}
	
	plotter.downloadPlot = function()
	{
		var plotUrl = plotter.generatePlotUrl(true);
		if (plotUrl)
		{
			var url = '../../plotter/download.jsp?ID='+App.getSessionId();
			url += '&plot='+encodeURIComponent(plotUrl);
			Dialogs.openPopup(url, 'DownloadPlot', 300, 200);
		}
	}

	plotter.savePlotAs = function()
	{
		var plotUrl = plotter.generatePlotUrl(true);
		if (plotUrl)
		{
			var url = '../../plotter/save_as.jsp?ID='+App.getSessionId();
			url += '&plot='+encodeURIComponent(plotUrl);
			Dialogs.openPopup(url, 'SavePlotAs', 450, 300);
		}
	}
	
	return plotter;
}();

Doc.onLoad(Plotter.initPage);
